### Install and Load Add-on Packages
# options(repos="https://cran.ism.ac.jp/")
# install.packages(c("dplyr", "ggplot2", "magrittr", "xtable", "flextable", "officer"), dep=T)
library(dplyr)
library(ggplot2)
library(magrittr)
library(knitr)
library(xtable)
library(flextable)
library(officer)



### Example: Create PowerPoint Document
myplot  <- ggplot(data=iris, aes(Sepal.Length, Sepal.Width)) + geom_point()
myppt   <- read_pptx()
( mylab <- layout_summary(myppt)[[1]] )    # Show Slide Layout Name
( mytmp <- layout_summary(myppt)[[2]][1] ) # Show Slide Master Name

myppt <- myppt %>%
  add_slide(layout="Title Slide", master=mytmp) %>%
  ph_with_text(type="ctrTitle", str="Iris Data") %>%
  ph_with_text(type="subTitle", str="Table & Figure") %>%
  ph_with_text(type="dt",       str=format(Sys.Date())) %>%
  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Scatter Plot") %>%
  ph_with_gg(value=myplot) %>%
  add_slide(layout="Two Content", master=mytmp) %>%
  ph_with_text(type="title", str="Table") %>%
  ph_with_table(type="body", value=iris[1:5,], index=1) %>%
  ph_with_text(type="body", str="Iris data", index=2)
print(myppt, target="c:/temp/sample.pptx")



### Example: Create WORD Document
myplot <- ggplot(data=iris, aes(Sepal.Length, Sepal.Width)) + geom_point()
mydoc  <- read_docx() %>% 
  body_add_par(value="Table of contents", style="heading 1") %>% 
  body_add_toc(level=2) %>% 
  body_add_break() %>% 
  body_add_par(value="Iris Data", style="heading 1") %>% 
  body_add_par(value="Table", style="heading 2") %>% 
  body_add_par("Sepal.Length vs. Sepal.Width", style="Normal") %>% 
  body_add_table(value=iris[1:5,], style="table_template" ) %>% 
  body_add_par(value="Scatter Plot", style="heading 2") %>% 
  body_add_gg(value=myplot, style="centered")
print(mydoc, target="c:/temp/sample.docx")



### Procedures
myppt   <- read_pptx()
( mylab <- layout_summary(myppt)[[1]] )    # Show Slide Layout Name
( mytmp <- layout_summary(myppt)[[2]][1] ) # Show Slide Master Name
myppt %>%
  layout_properties(layout="Title and Content", master=mytmp)%>%
  kable(digits=2)                          # Show Slide Properties



myppt   <- read_pptx("C:/temp/nds.pptx")
( mylab <- layout_summary(myppt)[[1]] )    # Show Slide Layout Name
( mytmp <- layout_summary(myppt)[[2]][1] ) # Show Slide Master Name
for (i in 1:length(mylab)) {
  layout_properties(myppt, mylab[i], master=mytmp) %>% kable(digits=2) %>% print()
}



myplot <- ggplot(data=iris, aes(Sepal.Length, Sepal.Width)) + geom_point()

myppt  <- myppt %>%

  # Title Slide
  add_slide(layout="Title Slide", master=mytmp) %>%
  ph_with_text(type="ctrTitle", str="Iris Data") %>%
  ph_with_text(type="subTitle", str="Table & Figure") %>%
  ph_with_text(type="dt",       str=format(Sys.Date())) %>%

  # Slide for Scatter Plot
  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Scatter Plot") %>%
  ph_with_gg(value=myplot) %>%

  # Slide for Text and Table
  add_slide(layout="Two Content", master=mytmp) %>%
  ph_with_text(type="title", str="Table") %>%
  ph_with_table(type="body", value=iris[1:5,], index=1) %>%
  ph_with_text(type="body", str="Iris data", index=2)

print(myppt, target="c:/temp/sample.pptx")



### Details
# Add Texts
myppt <- read_pptx()
mylab <- layout_summary(myppt)[[1]]
mytmp <- layout_summary(myppt)[[2]][1]

myppt <- myppt %>%
  add_slide(layout="Two Content", master=mytmp) %>%
  ph_with_text(type="title", str="A title") %>%
  ph_with_text(type="ftr",   str="A footnote") %>%
  ph_with_text(type="dt",    str=format(Sys.Date())) %>%
  ph_with_text(type="body",  str="A first text",  index=1) %>%
  ph_with_text(type="body",  str="A second text", index=2)
print(myppt, target="c:/temp/sample.pptx") 



myppt <- read_pptx()
mytmp <- layout_summary(myppt)[[2]][1]
myppt <- myppt %>%
  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="A title") %>%
  ph_with_ul(type="body", level_list=c(1,2), str_list=c("aaa","bbb"),
    style=fp_text(font.size=0, color="red")) %>%
    ph_add_par(type="body", level=3) %>%
    ph_add_text(type="body", str="ccc", style=fp_text(color="blue"))
print(myppt, target="c:/temp/sample.pptx") 



# Add Figures
png("C:/temp/myplot.png")
plot(iris$Sepal.Length, iris$Sepal.Width)
dev.off()
myplot <- ggplot(data=iris, aes(Sepal.Length, Sepal.Width)) + geom_point()
myppt  <- read_pptx()
mytmp  <- layout_summary(myppt)[[2]][1]

myppt <- myppt %>%
  # Add Plot by ph_with_img()
  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Scatter Plot by plot()") %>%
  ph_with_img(type="body", src="C:/temp/myplot.png", width=5, height=5) %>%

  # Add Plot by ph_with_img_at()
  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Scatter Plot by plot()") %>%
  ph_with_img_at(src="C:/temp/myplot.png", left=2, top=2, width=5, height=5) %>%

  # Add Plot by ph_with_gg()
  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Scatter Plot by ggplot()") %>%
  ph_with_gg(value=myplot)
print(myppt, target="c:/temp/sample.pptx")



# Add Tables
myppt  <- read_pptx()
mytmp  <- layout_summary(myppt)[[2]][1]

myppt <- myppt %>%
  add_slide(layout="Two Content", master=mytmp) %>%
  ph_with_text(type="title", str="Table 1") %>%
  ph_with_table(type="body", value=iris[1:5,], index=1) %>%
  ph_with_text(type="body", str="Iris data", index=2) %>%
  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Table 2") %>%
  ph_with_table_at(value=iris[1:5,], left=1, top=2, width=8, height=5)
print(myppt, target="c:/temp/sample.pptx")



myft <- regulartable(iris[49:52,c(5,1,2)]) # Create "flextable" object
myft <- theme_booktabs(myft)               # Change "flextable" theme
myft <- autofit(myft)                      # Adjust Cell Width and Height

myppt  <- read_pptx()
mytmp  <- layout_summary(myppt)[[2]][1]
myppt  <- myppt %>%
  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_flextable(myft, type="body") 
print(myppt, target="c:/temp/sample.pptx")



myft1 <- regulartable(iris[49:52,], col_keys=c("Species","XXX","Sepal.Length","Sepal.Width"))
myft2 <- theme_booktabs(myft1)
myft3 <- theme_box(myft1)
myft4 <- theme_tron(myft1)
myft5 <- theme_tron_legacy(myft1)
myft6 <- theme_vanilla(myft1)
myft7 <- theme_zebra(myft1)
myft8 <- empty_blanks(myft7)   # Set Blank Columns as Transparent



myft  <- regulartable(iris[49:52,], col_keys=c("Species","Sepal.Length","Sepal.Width"))
myft1 <- merge_v(myft, j=c(1,3))
myft1
myft2 <- merge_v(myft, ~ Species)
myft2



myft  <- regulartable(iris[49:52,], col_keys=c("Species","Sepal.Length","Sepal.Width"))
myft1 <- set_header_labels(myft, Species = "Species", 
           Sepal.Length="Length", Sepal.Width="Width") %>%
         theme_vanilla()
myft1
myft2 <- add_header(myft1, Species = "Species", 
           Sepal.Length="Sepal", Sepal.Width="Sepal", top=T) %>%
         theme_vanilla() %>% autofit() %>% 
         merge_h(part="header") %>% merge_v(part="header") %>%
         add_footer(Species="This is a footnote." ) %>%
         merge_at(i=1, j=1:3, part="footer")
myft2 



myft  <- regulartable(iris[49:52,], 
           col_keys=c("Species","Sepal.Length","Sepal.Width"))
myft1 <- autofit(myft)
myft1
myft2 <- width(myft, j=~ Species, width=2) %>%
         height_all(height=0.5) %>%
         height(i=2, height=1)
myft2



myft1 <- regulartable(iris[49:52,], 
           col_keys=c("Species","Sepal.Length","Sepal.Width"))
myft1
myft2 <- align(myft1, align="center", part="all") %>%       # Text Alignment
         bg(bg="cyan", part="body") %>%                     # Background Color
         bold(part="header") %>%                            # Bold
         color(color="red", part="header") %>%              # Font Color
         font(j="Species", fontname="Arial") %>%            # Font
         fontsize(part="all", size=14) %>%                  # Font Size
         italic(i=~ Sepal.Length>6,                         # Conditional format
                j=~ Sepal.Length+Sepal.Width, italic=T) %>% # Italic
         padding(padding=3, part="all")                     # Padding (space around a text in a cell)
myft2



# Create Border Properties Objects
b0 <- fp_border(color="white", style="none")
b1 <- fp_border(color="blue", style="solid", width=1)
b2 <- fp_border(color="red", style="dotted", width=2) # style="dashed" is available
b3 <- fp_border(color="black", width=3)

myft  <- regulartable(iris[49:52,], 
           col_keys=c("Species","Sepal.Length","Sepal.Width")) %>%
         add_footer(Species="This is a footnote." ) %>%
         merge_at(i=1, j=1:3, part="footer")
myft1 <- border_remove(myft)                         # Remove All Borders
myft1
myft2 <- hline(myft1, border=b1) %>%                 # Set horizontal lines
         hline_top(border=b2, part="body") %>%       # Set first horizontal line
         hline_bottom(border=b0, part="footer") %>%  # Set last horizontal line
         vline(border=b1, part="header") %>%         # Set vertical lines 
         vline_left(border=b0, part="footer") %>%    # Set first vertical line
         vline_right(border=b0, part="footer")       # Set last vertical line
myft2
myft3 <- border_inner_h(myft1, border=b1, part="all") %>%
         border_inner_v(border=b2, part="all") %>%
         border_outer(border=b3, part="all")
myft3



# Miscellaneous
b     <- fp_border(color="white", style="none")
myft  <- regulartable(iris[49:52,], 
           col_keys=c("Species","Sepal.Length","Sepal.Width"))
myft1 <- border_inner_h(myft, border=b, part="all") %>%
         border_inner_v(border=b, part="all")
myft1
b     <- update(b, color="blue", style="solid", width=1)
myft2 <- border_outer(myft1, border=b, part="all")
myft2



myppt <- read_pptx()
mylab <- layout_summary(myppt)[[1]]
mytmp <- layout_summary(myppt)[[2]][1]
myppt <- myppt %>%
  add_slide(layout="Title and Content", master=mytmp) %>%
  add_slide(layout="Title Only",        master=mytmp) %>%
  add_slide(layout="Two Content",       master=mytmp)       # Create 3 slides

myppt <- myppt %>% remove_slide(index=1)                    # Remove Slide #1
myppt <- myppt %>%
  on_slide(index=1) %>%                                     # Focus on #1
  ph_with_text(type="title", str="Dummy Title") %>%         # Add a text
  on_slide(index=2) %>%                                     # Focus on #2
  ph_with_table(type="body", value=iris[1:5,], index=1) %>% # Add a table
  ph_with_text(type="body", str="Iris data", index=2) %>%   # Add a text
  on_slide(index=1) %>%                                     # Focus on #1
  ph_remove(type="title")                                   # Remove title
print(myppt, target="c:/temp/sample.pptx") 



result  <- lm(Sepal.Length ~ Species, data=iris)
mytable <- xtable(summary(result)$coefficients)
myft1   <- xtable_to_flextable(mytable, hline.after=c(-1,0,1,nrow(mytable)))
myft1

mytable <- table(sample(LETTERS[1:3],10, rep=T))
myft2   <- xtable_to_flextable(xtable(mytable)) %>%
           set_header_labels(rowname="Char.", V1="Freq.") %>%
           autofit()
myft2



### Example: Define Functions
demog_summary <- function (var, label, digit=1) {
  eval(substitute(TMP  <- subset(ADSL, !is.na(var) & RANDFL=="Y")))
  eval(substitute(TMP1 <- summarise(group_by(TMP, TRT01P), N=n(), MEAN=mean(var), SD=sd(var), MIN=min(var), Q1=quantile(var, pr=0.25), MEDIAN=median(var), Q3=quantile(var, pr=0.75), MAX=max(var))))
  eval(substitute(TMP2 <- summarise(TMP, N=n(), MEAN=mean(var), SD=sd(var), MIN=min(var), Q1=quantile(var, pr=0.25), MEDIAN=median(var), Q3=quantile(var, pr=0.75), MAX=max(var))))
  TMP3      <- rbind(TMP1, data.frame(TRT01P="Total", TMP2))
  TMP4      <- data.frame(VAR=rep(label, nrow(TMP3)), TMP3)
  TMP4$MEAN <- formatC(TMP4$MEAN, digits=digit, format="f")
  TMP4$SD   <- formatC(TMP4$SD,   digits=digit+1, format="f")
  TMP4$Q1   <- formatC(TMP4$Q1,   digits=digit+1, format="f")
  TMP4$Q3   <- formatC(TMP4$Q3,   digits=digit+1, format="f")
  return(TMP4)
}

demog_freq <- function(var, label) {
  for (i in 1:length(var)) {
    eval(parse(text=paste("TMP  <- subset(ADSL, !is.na(", var[i], ") & RANDFL=='Y')")))
    eval(parse(text=paste("TMP1 <- as.data.frame(table.margins(xtabs(~", var[i], "+ TRT01P, data=TMP)))")))
    TMP2 <- data.frame(VAR=rep(label[i], nrow(TMP1)), CAT=row.names(TMP1), TMP1)[-nrow(TMP1),]
    if (i==1) RESULT <- TMP2
    else      RESULT <- rbind(RESULT, TMP2)
  }
  row.names(RESULT) <- NULL
  return(RESULT)
}

kmtable <- function (paramcd, pflag, times) {
  eval(substitute(TMP <- subset(ADTTE, PARAMCD==paramcd & pflag=="Y")))
  p_num <- pchisq(survdiff(Surv(AVAL, EVENT) ~ TRTPN, data=TMP)$chisq, 1, lower=F)
  p_chr <- ifelse(p_num<0.0001, "Logrank test: p<0.0001", paste0("Logrank test: p=",formatC(p_num, digits=4, format="f")))

  maxt <- max(times)
  TMP1 <- summary(survfit(Surv(AVAL, EVENT) ~ TRTPN, data=TMP, type="kaplan-meier", 
                          error="greenwood", conf.type="plain"), times=0:maxt)
  TMP2 <- data.frame(TIME=TMP1$time, N=TMP1$n.risk, EVENT=TMP1$n.event, 
                     CENSOR=TMP1$n.censor, ESTIMATE=100*(1-TMP1$surv), 
                     SE=100*(TMP1$std.err), GROUP=as.numeric(TMP1$strata))
  TMP3 <- list(); N <- list()
  for (i in 1:2) {
    KM   <- subset(TMP2, GROUP==i); MT <- max(KM$TIME); 
    LAST <- KM[MT+1,]; LAST$N <- LAST$EVENT <- LAST$CENSOR <- 0
    if (maxt > MT) {
      for (j in (MT+1):maxt) {
        LAST$TIME <- j
        KM        <- rbind(KM, LAST)
      }
    }
    KM <- transform(KM, EVENT  =cumsum(EVENT),
                        MARKER =CENSOR,
                        CENSOR =cumsum(CENSOR),
                        LOWERCI=(ESTIMATE-qnorm(0.975)*SE),
                        UPPERCI=(ESTIMATE+qnorm(0.975)*SE))
    KM$LOWERCI <- ifelse(round(KM$ESTIMATE,8)==100, 100, 
                  ifelse(KM$LOWERCI          <=  0,   0, KM$LOWERCI))
    KM$UPPERCI <- ifelse(round(KM$ESTIMATE,8)==100, 100, 
                  ifelse(KM$UPPERCI          >=100, 100, KM$UPPERCI))
    TMP3[[i]]  <- KM
    N[[i]]     <- subset(KM, TIME %in% times, select=N)[,, drop=T]
  }
  TMP4       <- rbind(TMP3[[1]],TMP3[[2]])
  TMP4$GROUP <- factor(TMP4$GROUP, level=1:2, label=c("ABC-XXX", "Placebo"))
  TMP5       <- subset(TMP4, TIME %in% times, select=c(7, 1:5, 9:10))
  row.names(TMP4) <- NULL; row.names(TMP5) <- NULL

  TMP5$N        <- formatC(TMP5$N,        digits=0, format="f")
  TMP5$EVENT    <- formatC(TMP5$EVENT,    digits=0, format="f")
  TMP5$CENSOR   <- formatC(TMP5$CENSOR,   digits=0, format="f")
  TMP5$ESTIMATE <- formatC(TMP5$ESTIMATE, digits=1, format="f")
  TMP5$UPPERCI  <- formatC(TMP5$UPPERCI,  digits=1, format="f")
  TMP5$LOWERCI  <- formatC(TMP5$LOWERCI,  digits=1, format="f")

  TMP6 <- regulartable(TMP5) %>%
          set_header_labels(GROUP="Group", TIME="Time", N="At Risk", EVENT="Event", 
            CENSOR="Censor", ESTIMATE="%", LOWERCI="Lower", UPPERCI="Upper") %>%
          add_header(GROUP="Group", TIME="Time", N="At Risk", EVENT="Event", 
            CENSOR="Censor", ESTIMATE="%", LOWERCI="95% CI", UPPERCI="95% CI", top=T) %>%
          merge_v(part="header") %>% merge_v(j=1) %>% merge_h(part="header") %>%
          align(align="center", part="all") %>% width(width=1.1) %>% height_all(0.4)

  if (p_num < 0.05) { P <- "Superiority of ABC-XXX to Placebo was confirmed.";     C="red"  }
  else              { P <- "Superiority of ABC-XXX to Placebo was not confirmed."; C="blue" }
  return( list(all=TMP4, lifetable=TMP5, times=times, n1=N[[1]], n2=N[[2]], p_num=p_num, p_chr=p_chr, table=TMP6, pot=P, col=C) )
}

kmplot <- function (km_obj, xlim=c(-35,max(km_obj$times)), xlab="Days") {
  ggplot(data=km_obj$all) +
    geom_step(mapping=aes(x=TIME, y=ESTIMATE, color=GROUP, size=GROUP)) + 
    geom_point(data=subset(km_obj$all, MARKER>0), mapping=aes(x=TIME, y=ESTIMATE, color=GROUP), shape=3, size=2) +
    xlab(xlab) + ylab("Cumulative Incidence (%)") + 
    scale_size_manual(values=c(1.5,0.5)) +
    scale_x_continuous(breaks=km_obj$times,        limits=xlim) +
    scale_y_continuous(breaks=seq(0, 100, by= 20), limits=c(-8, 100)) +
    annotate("text", x=20,           y=95, label=km_obj$p_chr, color="black", fontface=2) +
    annotate("text", x=xlim[1],      y=-4, label="ABC-XXX",    color="#F8766D") +
    annotate("text", x=km_obj$times, y=-4, label=km_obj$n1,    color="#F8766D") +
    annotate("text", x=xlim[1],      y=-8, label="Placebo",    color="#00BFC4") +
    annotate("text", x=km_obj$times, y=-8, label=km_obj$n2,    color="#00BFC4")
}

riskdiff <- function (correct=TRUE) {
  ALL <- list(); k <- 1
  for (i in c("GU","DU")) {
    TMP  <- subset(ADTTE, !is.na(CNSR) & FASFL=="Y" & PARAMCD==i)
    TMP1 <- as.data.frame(table.margins(xtabs(~ TRTP + CNSR, data=TMP)))
    TMP2 <- data.frame(VAR=rep(i, nrow(TMP1)), CAT=row.names(TMP1), TMP1)[-nrow(TMP1),]

    TMP2$Yes      <- TMP2$X0
    TMP2$No       <- TMP2$X1
    BT1           <- binom.test(TMP2$X0[1], TMP2$Total[1])
    BT2           <- binom.test(TMP2$X0[2], TMP2$Total[2])
    TMP2$ESTIMATE <- c(BT1$estimate, BT2$estimate)
    TMP2$LCL1     <- c(as.vector(BT1$conf.int[1]), as.vector(BT2$conf.int[1]))
    TMP2$UCL1     <- c(as.vector(BT1$conf.int[2]), as.vector(BT2$conf.int[2]))

    n             <- TMP2$Total
    p             <- TMP2$ESTIMATE
    TMP2$DIFF     <- c(p[1] - p[2], NA)
    se            <- sqrt(p[1] * (1 - p[1]) / n[1] + p[2] * (1 - p[2]) / n[2])
    TMP2$LCL2     <- c(p[1] - p[2] - qnorm(0.975) * se, NA)
    TMP2$UCL2     <- c(p[1] - p[2] + qnorm(0.975) * se, NA)

    chisq         <- chisq.test(TMP1[1:2,1:2], correct=correct)
    TMP2$DF       <- c(as.vector(chisq$parameter), NA)
    TMP2$CHISQ    <- c(as.vector(chisq$statistic), NA)
    TMP2$P        <- c(as.vector(chisq$p.value), NA)

    row.names(TMP2) <- NULL
    ALL[[k]]        <- TMP2
    k               <- k+1
  }
  TMP1       <- rbind(ALL[[1]],ALL[[2]])[,c(1:2, 6:7, 5, 8:16)]
  TMP1$Yes   <- paste0(TMP1$Yes, " (", formatC(TMP1$ESTIMATE*100, digits=1, format="f"), ")")
  TMP1$LCL1  <- formatC(TMP1$LCL1*100,     digits=1, format="f")
  TMP1$UCL1  <- formatC(TMP1$UCL1*100,     digits=1, format="f")
  TMP1$DIFF  <- formatC(TMP1$DIFF*100,     digits=1, format="f")
  TMP1$LCL2  <- formatC(TMP1$LCL2*100,     digits=1, format="f")
  TMP1$UCL2  <- formatC(TMP1$UCL2*100,     digits=1, format="f")
  TMP1$CHISQ <- formatC(TMP1$CHISQ, digits=3, format="f")
  TMP1$P     <- ifelse(TMP1$P<0.0001, "<0.0001", formatC(TMP1$P, digits=4, format="f"))

  TABLE1 <- regulartable(TMP1, col_keys=c("VAR", "CAT", "Yes", "No", "Total", "LCL1", "UCL1")) %>%
            set_header_labels(VAR="Test", CAT="Group", Yes="Yes (%)", No="No", Total="Total", 
              LCL1="Lower", UCL1="Upper") %>%
            add_header(VAR="Test", CAT="Group", Yes="Yes (%)", No="No", Total="Total", 
              LCL1="95% CI", UCL1="95% CI", top=T) %>%
            merge_v(part="header") %>% merge_v(j=1) %>% merge_h(part="header") %>%
            align(align="center", part="all") %>% width(width=1.1) %>% height_all(0.4)

  TMP1$CAT <- rep("ABC-XXX - Placebo", 4)
  TABLE2 <- regulartable(TMP1[c(1,3),], col_keys=c("VAR", "CAT", "DIFF", "LCL2", "UCL2", "CHISQ", "DF", "P")) %>%
            set_header_labels(VAR="Test", CAT="Group", DIFF="Diff.", LCL2="Lower", UCL2="Upper", 
              CHISQ="Chisq.", DF="DF", P="p-value") %>%
            add_header(VAR="Test", CAT="Group", DIFF="Diff.", LCL2="95% CI", UCL2="95% CI", 
              CHISQ="Chi-square Test", DF="Chi-square Test", P="Chi-square Test", top=T) %>%
            merge_v(part="header") %>% merge_h(part="header") %>%
            align(align="center", part="all") %>% width(width=c(1.1, 2.2, rep(0.9,6))) %>% height_all(0.4)

  if (TMP1[1,14] < 0.05) { P1 <- "The difference in proportions of GU was statistically significant.";     C1="red"  }
  else                   { P1 <- "The difference in proportions of GU was not statistically significant."; C1="blue" }
  if (TMP1[3,14] < 0.05) { P2 <- "The difference in proportions of DU was statistically significant.";     C2="red"  }
  else                   { P2 <- "The difference in proportions of DU was not statistically significant."; C2="blue" }
  return( list(table1=TABLE1,table2=TABLE2, pot1=P1, pot2=P2, col1=C1, col2=C2) )
}

covadjust <- function (paramcd, pflag, cov, label, ref) {
  eval(substitute(TMP <- subset(ADTTE, PARAMCD==paramcd & pflag=="Y")))
  eval(substitute(TMP1 <- as.data.frame(summary(coxph(Surv(AVAL, EVENT) ~ relevel(factor(TRTPN), ref=ref) + cov, data=TMP))$conf.int)[1,c(1,3,4)]))
  return( data.frame(VAR=label, TMP1) )
}

adjplot <- function (adj_obj, cex=1.5, xlim=c(0,1.5), breaks=seq(xlim[1], xlim[2], 0.5)) {
  DATA          <- rbind(rep(NA,3), adj_obj[1,2:4], rep(NA,3), adj_obj[2:nrow(adj_obj),2:4])
  adj_obj[,2:4] <- round(adj_obj[,2:4],2)
  TEXT          <- rbind(c("Covariate", "HR", "95% LCL", "95% UCL"),
                         adj_obj[1,], rep(NA,4), adj_obj[2:nrow(adj_obj),])
  row.names(DATA) <- NULL
  row.names(TEXT) <- NULL
  forestplot(TEXT, DATA, graph.pos=2, hrzl_lines=gpar(col="#444444"), xlog=FALSE, boxsize=0.2, 
    col=fpColors(box="royalblue",line="darkblue", summary="royalblue"), zero=1, 
    txt_gp=fpTxtGp(cex=cex), is.summary=c(T,rep(F,nrow(adj_obj)+1)), clip=xlim, xticks=breaks)
}

subgroup <- function (paramcd, pflag, cov, label, sub, sublabel, ref) {
  eval(substitute(TMP <- subset(ADTTE, PARAMCD==paramcd & pflag=="Y" & cov==sub)))
  eval(substitute(TMP1 <- as.data.frame(summary(coxph(Surv(AVAL, EVENT) ~ relevel(factor(TRTPN), ref=ref), data=TMP))$conf.int)[1,c(1,3,4)]))
  return( data.frame(VAR=label, CAT=sublabel, TMP1) )
}

subplot <- function (sub_obj, cex=1.5, xlim=c(0,1.5), breaks=seq(xlim[1], xlim[2], 0.5)) {
  DATA          <- rbind(rep(NA,3), sub_obj[1,3:5], rep(NA,3), sub_obj[2:nrow(sub_obj),3:5])
  sub_obj[,3:5] <- round(sub_obj[,3:5],2)
  TEXT          <- rbind(c("Subgroup", "Category", "HR", "95% LCL", "95% UCL"),
                         sub_obj[1,], rep(NA,5), sub_obj[2:nrow(sub_obj),])
  row.names(DATA) <- NULL
  row.names(TEXT) <- NULL
  forestplot(TEXT, DATA, graph.pos=3, hrzl_lines=gpar(col="#444444"), xlog=FALSE, boxsize=0.2, 
    col=fpColors(box="royalblue",line="darkblue", summary="royalblue"), zero=1, 
    txt_gp=fpTxtGp(cex=cex), is.summary=c(T,rep(F,nrow(sub_obj)+1)), clip=xlim, xticks=breaks)
}

teae <- function (rel=FALSE, sae=FALSE) {
  N <- xtabs(~ TRT01A, data=subset(ADSL, SAFFL=="Y"))
  if (rel) TMP1 <- subset(ADAE, SAFFL=="Y" & TRTEMFL=="Y" & ARELN==1)
  else     TMP1 <- subset(ADAE, SAFFL=="Y" & TRTEMFL=="Y")
  if (sae) TMP1 <- subset(TMP1, AESER=="Y")
  TMP2  <- subset(distinct(TMP1, SUBJID, .keep_all=T), select=c(SAFFL, TRTA))
  TMP3  <- as.data.frame(table.margins(xtabs(~ SAFFL + TRTA, data=TMP2)))
  TMP4  <- data.frame(PT="Subjects with Any TEAEs", TMP3)[-nrow(TMP3),]

  TMP5  <- subset(distinct(TMP1, SUBJID, AEDECOD, .keep_all=T), select=c(AEDECOD, TRTA))
  TMP6  <- as.data.frame(table.margins(xtabs(~ AEDECOD + TRTA, data=TMP5)))
  TMP7  <- data.frame(PT=row.names(TMP6), TMP6)[-nrow(TMP6),]
  TMP8  <- transform(arrange(rbind(TMP4, TMP7), desc(Total)), P1=ABC.XXX/N[1], P2=Placebo/N[2])
  row.names(TMP8) <- NULL
  TMP8$PT         <- as.character(TMP8$PT)
  TMP9  <- transform(TMP8, DIFF=(P1-P2), SE=sqrt(P1*(1-P1)/N[1]+P2*(1-P2)/N[2]))
  TMP10 <- transform(TMP9, LCL=(DIFF-qnorm(0.975)*SE), UCL=(DIFF+qnorm(0.975)*SE), NO=1:nrow(TMP9))[,-8]
  return( list(table=TMP10, n=N) )
}

aeplot <- function (ae_obj, cex=1.5, xlim=c(-20,20), breaks=seq(xlim[1], xlim[2], 5)) {
  DATA <- 100*rbind(rep(NA,3), rep(NA,3), ae_obj$table[1,], rep(NA,3), ae_obj$table[2:nrow(ae_obj$table),])[,7:9]
  TMP1 <- transform(ae_obj$table, G1=paste0(ABC.XXX, " (", format(round(P1*100,1), nsmall=1), ")"), 
                                  G2=paste0(Placebo, " (", format(round(P2*100,1), nsmall=1), ")"), 
                                  DF=paste0(format(round(DIFF*100,1), nsmall=1), "%"),
                                  stringsAsFactors=F)[,c(1,11:13)]
  TEXT <- rbind(c("", "ABC-XXX", "Placebo", ""),
                c("PT Name", "n (%)", "n (%)", "Difference"),
                TMP1[1,], rep(NA,4), TMP1[2:nrow(TMP1),])
  row.names(DATA) <- NULL
  row.names(TEXT) <- NULL
  forestplot(TEXT, DATA, hrzl_lines=gpar(col="#444444"), xlog=FALSE, boxsize=0.2, 
    col=fpColors(box="royalblue",line="darkblue", summary="royalblue"), 
    txt_gp=fpTxtGp(cex=cex), is.summary=c(T,T,rep(F,nrow(ae_obj$table)+1)), clip=xlim, xticks=breaks)
}

teae_intensity <- function (rel=FALSE) {
  N    <- xtabs(~ TRT01A, data=subset(ADSL, SAFFL=="Y"))
  if (rel) TMP1 <- subset(ADAE, SAFFL=="Y" & TRTEMFL=="Y" & ARELN==1)
  else     TMP1 <- subset(ADAE, SAFFL=="Y" & TRTEMFL=="Y")
  TMP2  <- arrange(TMP1, SUBJID, desc(ASEV))
  TMP3  <- subset(distinct(TMP2, SUBJID, .keep_all=T), select=c(TRTA, ASEV))
  TMP4  <- as.data.frame(table.margins(xtabs(~ ASEV + TRTA, data=TMP3)))
  TMP5  <- data.frame(PT="Subjects with Any TEAEs", Intensity=row.names(TMP4), TMP4)[-nrow(TMP4),]

  TMP6  <- arrange(TMP1, SUBJID, AEDECOD, desc(ASEV))
  TMP7  <- subset(distinct(TMP6, SUBJID, AEDECOD, .keep_all=T), select=c(TRTA, AEDECOD, ASEV))
  TMP8  <- as.data.frame(ftable(TMP7$ASEV, TMP7$AEDECOD, TMP7$TRTA))
  TMP9  <- dcast(TMP8, Var2 + Var1 ~ Var3, value.var="Freq")
  TMP10 <- transform(data.frame(PT=TMP9[,1], Intensity=TMP9[,2], ABC.XXX=TMP9[,3], Placebo=TMP9[,4]),
                     Total=(ABC.XXX+Placebo))
  TMP11 <- rbind(TMP5, TMP10)
  TMP11$PT         <- as.character(TMP11$PT)
  TMP11$Intensity  <- as.character(TMP11$Intensity)

  ALL              <- teae()$table
  ALL$Intensity    <- rep("", nrow(ALL))
  TMP11            <- full_join(TMP11, select(ALL, PT, NO), by="PT")
  TMP12            <- rbind(TMP11, select(ALL, PT, Intensity, ABC.XXX, Placebo, Total, NO))
  TMP12            <- arrange(TMP12, NO, Intensity)
  TMP12$Intensity  <- ifelse(TMP12$Intensity != "", TMP12$Intensity, "Overall")
  row.names(TMP12) <- NULL
  TMP13            <- transform(TMP12, P1=ABC.XXX/N[1], P2=Placebo/N[2])[,]
  return( list(table=TMP13[,c(1:5,7:8)], n=N) )
}



### Example: Read ADaM Datasets and Create R Data Frames
# install.packages(c("epiR","epitools","forestplot","haven","reshape2"), dep=T)
library(dplyr)
library(epiR)
library(epitools)
library(forestplot)
library(ggplot2)
library(haven)
library(magrittr)
library(reshape2)
library(knitr)
library(flextable)
library(officer)

ADAE          <- read_sas("C:/temp/adae.sas7bdat")
ADSL          <- read_sas("C:/temp/adsl.sas7bdat")
ADTTE         <- read_sas("C:/temp/adtte.sas7bdat")
ADSL$AGEGR1   <- factor(ADSL$AGEGR1,   levels=c("Min<= - <65","65<= - <=Max"))
ADSL$ALCOHOL  <- factor(ADSL$ALCOHOL,  levels=c("Yes","No"))
ADSL$BMIGR1   <- factor(ADSL$BMIGR1,   levels=c("Min<= - <25.0","25.0<= - <=Max"))
ADSL$CAFFEINE <- factor(ADSL$CAFFEINE, levels=c("Yes","No"))
ADSL$HPYLORI  <- factor(ADSL$HPYLORI,  levels=c("Positive","Negative"))
ADSL$SEX      <- factor(ADSL$SEX,      levels=c("M","F"), labels=c("Male","Female"))
ADSL$TOBACCO  <- factor(ADSL$TOBACCO,  levels=c("Yes","No"))
ADTTE$EVENT   <- 1-ADTTE$CNSR

DEMOG1 <-               demog_summary(AGE  , "Age (years)",  1)
DEMOG1 <- rbind(DEMOG1, demog_summary(BMIBL, "BMI (kg/m^2)", 2))
DEMOG_TABLE1 <- regulartable(DEMOG1) %>%
                set_header_labels(VAR="Variable", TRT01P="Group",
                  N="N", MEAN="Mean", SD="S.D.", MIN="Min.",
                  Q1="Q1",MEDIAN ="Median", Q3="Q3", MAX="Max.") %>%
                merge_v(j=1) %>% width(width=0.9) %>% height_all(0.5)

DEMOG2         <- demog_freq(c("AGEGR1","ALCOHOL","BMIGR1","CAFFEINE","HPYLORI","SEX","TOBACCO"), 
                             c("Age (years)","Alcohol Consumption","BMI (kg/m^2)","Caffeine Consumption",
                               "H. pylori Infection","Gender","Tobacco Consumption"))
N              <- xtabs(~ TRT01P, data=subset(ADSL, RANDFL=="Y"))
DEMOG2$ABC.XXX <- paste0(DEMOG2$ABC.XXX, " (", formatC(DEMOG2$ABC.XXX/N[1]*100, digits=1, format="f"), ")")
DEMOG2$Placebo <- paste0(DEMOG2$Placebo, " (", formatC(DEMOG2$Placebo/N[2]*100, digits=1, format="f"), ")")
DEMOG_TABLE2   <- regulartable(DEMOG2) %>%
                  set_header_labels(VAR="Variable", CAT="Category",
                    ABC.XXX=paste0("[n=", N[1],   "]"),
                    Placebo=paste0("[n=", N[2],   "]"),
                    Total  =paste0("[n=", sum(N), "]")) %>%
                  add_header(VAR="Variable", CAT="Category",
                    ABC.XXX="ABC.XXX", Placebo="Placebo", Total="Total", top=T) %>%
                  merge_v(part="header") %>% merge_v(j=1) %>% 
                  align(align="center", part="all") %>% width(width=1.7) %>% height_all(0.35)

T1 <- kmtable("GU", FASFL,   c(0,100,200,300,400))
T2 <- kmtable("GU", PPROTFL, c(0,100,200,300,400))
T3 <- kmtable("DU", FASFL,   c(0,100,200,300,400))

ADDITIONAL <- riskdiff()

ADJUST <-               covadjust("GU", FASFL, 1       , "Overall", 2)
ADJUST <- rbind(ADJUST, covadjust("GU", FASFL, AGE     , "Age", 2))
ADJUST <- rbind(ADJUST, covadjust("GU", FASFL, SEX     , "Gender", 2))
ADJUST <- rbind(ADJUST, covadjust("GU", FASFL, BMIBL   , "BMI", 2))
ADJUST <- rbind(ADJUST, covadjust("GU", FASFL, HPYLORI , "H. pylori Infection", 2))
ADJUST <- rbind(ADJUST, covadjust("GU", FASFL, CAFFEINE, "Caffeine Consumption", 2))
ADJUST <- rbind(ADJUST, covadjust("GU", FASFL, ALCOHOL , "Alcohol Consumption", 2))
ADJUST <- rbind(ADJUST, covadjust("GU", FASFL, TOBACCO , "Tobacco Consumption", 2))
row.names(ADJUST) <- NULL
ADJUST$VAR        <- as.character(ADJUST$VAR)

SUB <-            subgroup("GU", FASFL, FASFL   , "Overall", "Y", "", 2)
SUB <- rbind(SUB, subgroup("GU", FASFL, AGEGR1N , "Age", 1, "Min<= - <65", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, AGEGR1N , "Age", 2, "65<= - <=Max", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, SEX     , "Gender", "M", "Male", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, SEX     , "Gender", "F", "Female", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, BMIGR1N , "BMI", 1, "Min<= - <25", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, BMIGR1N , "BMI", 2, "25<= - <=Max", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, HPYLORI , "H. pylori Infection", "Positive", "Positive", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, HPYLORI , "H. pylori Infection", "Negative", "Negative", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, CAFFEINE, "Caffeine Consumption", "Yes", "Yes", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, CAFFEINE, "Caffeine Consumption", "No", "No", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, ALCOHOL , "Alcohol Consumption", "Yes", "Yes", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, ALCOHOL , "Alcohol Consumption", "No", "No", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, TOBACCO , "Tobacco Consumption", "Yes", "Yes", 2))
SUB <- rbind(SUB, subgroup("GU", FASFL, TOBACCO , "Tobacco Consumption", "No", "No", 2))
row.names(SUB) <- NULL
SUB$VAR        <- as.character(SUB$VAR)
SUB$CAT        <- as.character(SUB$CAT)

PT_NUM            <- 4   # The number of Displayed PT per page
INT               <- teae_intensity()
N                 <- INT$n
INT$table$ABC.XXX <- paste0(INT$table$ABC.XXX, " (", formatC(INT$table$P1*100, digits=1, format="f"), ")")
INT$table$Placebo <- paste0(INT$table$Placebo, " (", formatC(INT$table$P2*100, digits=1, format="f"), ")")
INT               <- INT$table[,1:4]
x                 <- ceiling((nrow(INT)-4)/(4*PT_NUM))
INT_TABLE         <- list()
for (i in 1:x) {
  if (i < x || (i == x && (nrow(INT)-4)%%(4*PT_NUM)==0)) {
    TMP <- INT[c(1:4,(4+(4*PT_NUM)*(i-1)+1):(4+(4*PT_NUM)*(i-1)+(4*PT_NUM))),]
    TMP <- regulartable(TMP) %>%
           set_header_labels(PT="PT Name", Intensity="Intensity", 
             ABC.XXX=paste0("[n=", N[1],   "]"),
             Placebo=paste0("[n=", N[2],   "]")) %>%
           add_header(PT="PT Name", Intensity="Intensity", ABC.XXX="ABC-XXX", Placebo="Placebo", top=T) %>%
           merge_v(part="header") %>% merge_v(j=1) %>%
           hline(border=fp_border(color="black", width=2), i=4) %>%
           align(align="center", part="all") %>% align(j=1, align="left", part="body") %>%
           width(width=c(4,rep(1.5,3))) %>% height_all(0.25)
    INT_TABLE[[i]]   <- TMP
  } else {
    TMP <- INT[c(1:4,(4+(4*PT_NUM)*(i-1)+1):(4+(4*PT_NUM)*(i-1)+((nrow(INT)-4)%%(4*PT_NUM)))),]
    TMP <- regulartable(TMP) %>%
           set_header_labels(PT="PT Name", Intensity="Intensity", 
             ABC.XXX=paste0("[n=", N[1],   "]"),
             Placebo=paste0("[n=", N[2],   "]")) %>%
           add_header(PT="PT Name", Intensity="Intensity", ABC.XXX="ABC-XXX", Placebo="Placebo", top=T) %>%
           merge_v(part="header") %>% merge_v(j=1) %>%
           hline(border=fp_border(color="black", width=2), i=4) %>%
           align(align="center", part="all") %>% align(j=1, align="left", part="body") %>%
           width(width=c(4,rep(1.5,3))) %>% height_all(0.25)
    INT_TABLE[[i]]   <- TMP
  }
}

bmp(filename="C:/temp/adj.bmp", width=960, height=480)
  adjplot(ADJUST, cex=1.5)
  dev.off()
bmp(filename="C:/temp/sub.bmp", width=960, height=480)
  subplot(SUB, cex=1.3)
  dev.off()
bmp(filename="C:/temp/teae1.bmp", width=960, height=480)
  ALL <- teae()
  aeplot(ALL, cex=1.2, xlim=c(-15,10))
  dev.off()
bmp(filename="C:/temp/teae2.bmp", width=960, height=480)
  ALL <- teae(rel=TRUE)
  aeplot(ALL, cex=1.3, xlim=c(-15,10))
  dev.off()
bmp(filename="C:/temp/teae3.bmp", width=960, height=480)
  ALL <- teae(sae=TRUE)
  aeplot(ALL, cex=1.5, xlim=c(-15,10))
  dev.off()



### Example: Create PowerPoint Slides
# For Page Number
myPageNumber <- 0
pageNum <- function (x) {
  if (missing(x)) { myPageNumber <<- myPageNumber+1; return(as.character(myPageNumber)) }
  else            { myPageNumber <<- x;              return(as.character(myPageNumber)) }
}

# Create Slides
myppt <- read_pptx("C:/temp/nds.pptx")
mytmp <- layout_summary(myppt)[[2]][1]
myppt <- myppt %>%
  add_slide(layout="Title Slide", master=mytmp) %>%
  ph_with_text(type="ctrTitle", str="ABC-XXX Phase 3 Study") %>%
  ph_with_text(type="subTitle", str="Flash Result") %>%
  ph_with_text(type="ftr",       str=format(Sys.Date())) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Demographics and Baseline Characteristics") %>%
  ph_with_flextable_at(value=DEMOG_TABLE1, left=0.5, top=1.3) %>%
  ph_with_text(type="sldNum", str=pageNum(2)) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Demographics and Baseline Characteristics") %>%
  ph_with_flextable_at(value=DEMOG_TABLE2, left=0.5, top=1.3) %>%
  ph_with_text(type="sldNum", str=pageNum()) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Cumulative Incidence of GU [FAS]") %>%
  ph_with_gg_at(value=kmplot(T1), width=9, height=5, left=0.5, top=2) %>%
  ph_with_ul(type="body", level_list=1, str_list=T1$pot, style=fp_text(font.size=24, color=T1$col)) %>%
  ph_with_text(type="sldNum", str=pageNum()) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Life Table of GU [FAS]") %>%
  ph_with_flextable_at(value=T1$table, left=0.5, top=1.3) %>%
  ph_with_text(type="sldNum", str=pageNum()) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Cumulative Incidence of GU [PPS]") %>%
  ph_with_gg_at(value=kmplot(T2), width=9, height=5, left=0.5, top=2) %>%
  ph_with_ul(type="body", level_list=1, str_list=T2$pot, style=fp_text(font.size=24, color=T2$col)) %>%
  ph_with_text(type="sldNum", str=pageNum()) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Life Table of GU [PPS]") %>%
  ph_with_flextable_at(value=T2$table, left=0.5, top=1.3) %>%
  ph_with_text(type="sldNum", str=pageNum()) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Cumulative Incidence of DU [FAS]") %>%
  ph_with_gg_at(value=kmplot(T3), width=9, height=5, left=0.5, top=2) %>%
  ph_with_ul(type="body", level_list=1, str_list=T3$pot, style=fp_text(font.size=24, color=T3$col)) %>%
  ph_with_text(type="sldNum", str=pageNum()) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Life Table of DU [FAS]") %>%
  ph_with_flextable_at(value=T3$table, left=0.5, top=1.3) %>%
  ph_with_text(type="sldNum", str=pageNum()) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Proportion of GU/DU [FAS]") %>%
  ph_with_ul(type="body", level_list=1, str_list=ADDITIONAL$pot1,
    style=fp_text(font.size=20, color=ADDITIONAL$col1)) %>%
  ph_add_par(type="body", level=1) %>%
  ph_add_text(type="body", str=ADDITIONAL$pot2, 
    style=fp_text(font.size=20, color=ADDITIONAL$col2)) %>%
  ph_with_flextable_at(value=ADDITIONAL$table1, left=0.5, top=2.4) %>%
  ph_with_flextable_at(value=ADDITIONAL$table2, left=0.5, top=5.2) %>%
  ph_with_text(type="sldNum", str=pageNum()) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Covariate Adjusted Analysis") %>%
  ph_with_img_at(src="C:/temp/adj.bmp", left=0.4, top=1.1, width=9, height=6) %>%
  ph_with_text(type="sldNum", str=pageNum()) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Subgroup Analysis") %>%
  ph_with_img_at(src="C:/temp/sub.bmp", left=0.4, top=1.1, width=9, height=6) %>%
  ph_with_text(type="sldNum", str=pageNum()) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="All TEAEs") %>%
  ph_with_img_at(src="C:/temp/teae1.bmp", left=0.4, top=1.1, width=9, height=6) %>%
  ph_with_text(type="dt", str="TEAE: Treatment Emergent Adverse Event") %>%
  ph_with_text(type="sldNum", str=pageNum()) %>%

  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="Drug-related TEAEs") %>%
  ph_with_img_at(src="C:/temp/teae2.bmp", left=0.4, top=1.1, width=9, height=6) %>%
  ph_with_text(type="dt", str="TEAE: Treatment Emergent Adverse Event") %>%
  ph_with_text(type="sldNum", str=pageNum())

for (i in 1:length(INT_TABLE)) {
  myppt <- myppt %>%
    add_slide(layout="Title and Content", master=mytmp) %>%
    ph_with_text(type="title", str="Intensity of TEAEs") %>%
    ph_with_flextable_at(value=INT_TABLE[[i]], left=0.5, top=1.3) %>%
    ph_with_text(type="dt", str="TEAE: Treatment Emergent Adverse Event") %>%
    ph_with_text(type="sldNum", str=pageNum())
}

myppt <- myppt %>%
  add_slide(layout="Title and Content", master=mytmp) %>%
  ph_with_text(type="title", str="SAEs") %>%
  ph_with_img_at(src="C:/temp/teae3.bmp", left=0.4, top=1.1, width=9, height=6) %>%
  ph_with_text(type="sldNum", str=pageNum())
print(myppt, target="c:/temp/result.pptx")
